//=====================================================================
//	INCLUDES
//=====================================================================
//
#include <stdlib.h>

#include <vcl\vcl.h>
#pragma hdrstop

#ifdef _WIN32
	 #include <windows.h>
#endif
#include "GL/gl.h"

#include "triangle.h"
//---------------------------------------------------------------------
#pragma resource "*.dfm"
TOuterForm *OuterForm;

//=====================================================================
//	public Methoden
//=====================================================================
//
// Konstruktor
__fastcall TOuterForm::TOuterForm(TComponent* Owner)
	: TForm(Owner)
{
   GetOpenGLRC();                    // passende Eigenschaften suchen
   if(OpenGLCap)
     readyToDraw = true;
   else {
     readyToDraw = false;
		  MessageBox(NULL, "Exit program !"
                      "No OpenGL capabilities found.",
                      "Error", MB_OK);
   }

}

//---------------------------------------------------------------------
// Destruktor
__fastcall TOuterForm::~TOuterForm()
{

}

//---------------------------------------------------------------------
// OpenGL mit Fenster verbinden
void  __fastcall TOuterForm::GetOpenGLRC()
{
  OpenGLCap = false;                 // OpenGL-Faehigkeit wird nicht erwartet

  PIXELFORMATDESCRIPTOR pixelFormat =
  {
			sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
   	1,                              // Version of this structure
     PFD_SUPPORT_OPENGL |            // Support OpenGL calls in window
     PFD_DRAW_TO_WINDOW |            // Draw to Window (not to bitmap)
     PFD_DOUBLEBUFFER,               // Double buffered mode
     PFD_TYPE_RGBA,                  // RGBA Color mode
     16,                             // prefered color depth
     0, 0, 0, 0, 0, 0,               // color bits (ignored)
     0,                              // no alpha buffer
     0,                              // alpha bits (ignored)
     0,                              // no accumulation buffer
     0, 0, 0, 0,                     // accum bits (ignored)
     16,                             // Size of depth buffer
     0,                              // no stencil buffer
     0,                              // no auxiliary buffers
     PFD_MAIN_PLANE,                 // Draw in main plane (main layer)
     0,                              // reserved (fuer schlechte Zeiten ;-))
     0, 0, 0  };                     // no layer, visible, damage masks

  // Handle ist vom Eltern-Widget
  winDeviceContext = GetDC( Handle );

  // Schau'n, ob man die gewuenschten Eigenschaften bekommt
  int pixelFormatNr = ChoosePixelFormat( winDeviceContext,
  																		   &pixelFormat);
  if( pixelFormatNr == 0 )  {
  		MessageBox(NULL, "ChoosePixelFormat() failed:  "
               	     "Cannot find a suitable pixel format.",
                 	   "Error", MB_OK);
  }
  // ansonsten versuchen, das Ding zu setzen
  else if( SetPixelFormat( winDeviceContext,
  													pixelFormatNr,
                           &pixelFormat      ) == FALSE )  {
		 MessageBox(NULL, "SetPixelFormat() failed:  "
                     "Cannot set format specified.",
                     "Error", MB_OK);
  }
	 else {                               // wir haben was passendes gefunden
  	 // endgueltig verbinden
    glRenderingContext = wglCreateContext( winDeviceContext );
    wglMakeCurrent( winDeviceContext, glRenderingContext );

    // die Faehigkeit festhalten und auf DoubleBuffer checken
    OpenGLCap = true;
		 //GLboolean  *params ; // dat geht nich bei Version GLU 1.0 :-(
		 //glGetBooleanv(GL_DOUBLEBUFFER, params);
		 doubleBuffer = true; // pauschale Annahme
	 }
}

//=====================================================================
//	Callbacks
//=====================================================================
//
// Systemaufruf fuer Neuzeichnen nach Ueberdeckung
void __fastcall TOuterForm::Paint()
{
   GLNewDraw();
}

//---------------------------------------------------------------------
// Systemaufruf fuer Neuzeichnen nach Groessenaenderung
void __fastcall TOuterForm::Resize()
{
   int w, h;

   w = OuterForm->ClientWidth;
   h = OuterForm->ClientHeight;

   glViewport(0, 0, w, h);       /* Establish viewing area to cover entire window. */
   glMatrixMode(GL_PROJECTION);  /* Start modifying the projection matrix. */
   glLoadIdentity();             /* Reset project matrix. */
   glOrtho(0, w, 0, h, -1, 1);   /* Map abstract coords directly to window coords. */
   glScalef(1, -1, 1);           /* Invert Y axis so increasing Y goes down. */
   glTranslatef(0, -h, 0);       /* Shift origin up to upper-left corner. */

   GLNewDraw();
}

//=====================================================================
//	Methoden und Funktionsaufrufe
//=====================================================================
//
// das eigentliche Zeichnen
void __fastcall TOuterForm::GLNewDraw()
{
  if(readyToDraw) {
    readyToDraw = false;                 // neuer Durchlauf hat begonnen
    glClear(   GL_COLOR_BUFFER_BIT       // Zeichenflaeche loeschen
             | GL_DEPTH_BUFFER_BIT);     // Z-Buffer loeschen


    // a simple triangle without any transformations
    glBegin(GL_TRIANGLES);
      glColor3f(0.0, 0.0, 1.0);  /* blue */
      glVertex2i(20, 50);
      glColor3f(0.0, 1.0, 0.0);  /* green */
      glVertex2i(200, 100);
      glColor3f(1.0, 0.0, 0.0);  /* red */
      glVertex2i(100, 200);
    glEnd();

    // Zeichnung fertigstellen (bei Double-Buffer: nach vorn bringen)
    if( doubleBuffer )
      SwapBuffers(winDeviceContext);
    else
      glFlush();             // so schnell wie moeglich zeichnen

    readyToDraw = true;
  } // Durchlauf beendet
}

//---------------------------------------------------------------------
// ExitDialog und Destruktor
void __fastcall TOuterForm::ProgrammStop1Click(TObject *Sender)
{
   exit(0);
}



